@@ -228,7 +228,9 @@ class PonderTransformer(nn.Module):
             halting_logits = []
             layer_halt = []
 
-            for _ in range(self.train_max_steps):
+            for i in range(self.train_max_steps):
+                is_last = i == (self.train_max_steps - 1)
+
                 x, halt_logits = self.block(x)
                 hiddens.append(x)
 
@@ -240,16 +242,22 @@ class PonderTransformer(nn.Module):
                 # calculating halting probs
 
                 halting_probs = torch.stack(halting_logits, dim = 1).sigmoid()
-                p = calc_geometric(halting_probs, dim = 1)
-                should_halt = (torch.rand_like(p) <= p)[:, -1]
+                p = calc_geometric(halting_probs, dim = 1)[:, -1]
+                should_halt = torch.rand_like(p) <= p
 
                 # stack the halting signal across layers and determine whether to stop early
 
                 layer_halt.append(should_halt)
-                layer_was_halted = torch.any(torch.stack(layer_halt), dim = 0)
+
+                # do not exit early if it is the last one
+
+                if is_last:
+                    continue
 
                 # break if halting has been sampled for all layers
 
+                layer_was_halted = torch.any(torch.stack(layer_halt), dim = 0)
+
                 if torch.all(layer_was_halted):
                     break
 
